//----------------------------------------------------------------------
// File : Sample.hlsl
// Desc : Sample Shader
// Copyright(c) Project Asura. All right reserved.
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// Constant Buffer Variables
//----------------------------------------------------------------------
const float FLOT32_MAX = 3.402823466e+38f;

// eNX`
Texture2D decalMap : register( t0 );

#if 1
// ]ꂽ̂gꍇ
SamplerState decalSmp : register( s0 );
#else
// t@Cɒ`ꂽ̂gꍇ
SamplerState decalSmp {
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU = WRAP;
	AddressV = WRAP;
	AddressW = WRAP;
	MipmapLODBias = 0; 
	MaxAnisotropy = 1;
	ComparisonFunc = NEVER;
	MinLOD = 0;
	MaxLOD = FLOAT32_MAX;
};
#endif


////////////////////////////////////////////////////////////////////////
// VSInput structure
////////////////////////////////////////////////////////////////////////
struct VSInput
{
	float4 Position : POSITION;
	float2 TexCoord : TEXCOORD;
};

////////////////////////////////////////////////////////////////////////
// GSPSInput structure
////////////////////////////////////////////////////////////////////////
struct GSPSInput
{
	float4 Position : SV_POSITION;
	float2 TexCoord : TEXCOORD;
};

//----------------------------------------------------------------------
// Name : VSFunc()
// Desc : Vertex Shader Entry Point
//----------------------------------------------------------------------
GSPSInput VSFunc( VSInput input )
{
	GSPSInput output = (GSPSInput)0;

	// ̂܂ܑ
	output.Position = input.Position;
	output.TexCoord = input.TexCoord;

	return output;
}

//----------------------------------------------------------------------
// Name : GSFunc()
// Desc : Geometry Shader Entry Point
//----------------------------------------------------------------------
[maxvertexcount(3)]
void GSFunc( triangle GSPSInput input[ 3 ], inout TriangleStream<GSPSInput> stream )
{
	GSPSInput output;

	// H
	for( int i=0; i<3; ++i )
	{
		output.Position = input[ i ].Position;
		output.TexCoord = input[ i ].TexCoord;
		stream.Append( output );
	}
	stream.RestartStrip();
}


//----------------------------------------------------------------------
// Name : PSFunc()
// Desc : Pixel Shader Entry Point
//----------------------------------------------------------------------
float4 PSFunc( GSPSInput input ) : SV_TARGET0
{
	// o̓J[
	float4 output = float4( 0.0, 0.0, 0.0, 0.0 );

#if 1
	// Tv[Xe[ggꍇ
	{
		output = decalMap.Sample( decalSmp, input.TexCoord );
	}
#else
	// Tv[Xe[ggȂꍇ
	{
		uint width;
		uint levels;

		// 摜ƃ~bv}bvx擾
		decalMap.GetDimensions( width, levels );

		// KꂽeNX`W摜ňL΂
		float3 texCoord = float3( input.TexCoord * width, 0 );

		// eNZ擾
		output = decalMap.Load( texCoord );
	}
#endif

	return output;
}