//----------------------------------------------------------------------
// File : main.cpp
// Desc : Application Main Entry Point
// Copyright(c) Project Asura. All right reserved.
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// History
//----------------------------------------------------------------------
// <---> 2010/07/23 [Pocol] VK쐬
// <End>


//----------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------
#include <windows.h>
#include <d3d11.h>
#include <d3dx11.h>
#include <d3dcompiler.h>
#include <xnamath.h>
#include <cstdio>


//----------------------------------------------------------------------
// Linker
//----------------------------------------------------------------------
#pragma comment( lib, "d3d11.lib" )

#if defined(DEBUG) || defined(_DEBUG)
	#pragma comment( lib, "d3dx11d.lib" )
#else
	#pragma comment( lib, "d3dx11.lib" )
#endif


//----------------------------------------------------------------------
// Data Definitions
//----------------------------------------------------------------------
struct CustomVertex
{
	XMFLOAT3 position;	// _W
	XMFLOAT2 texcoord;	// eNX`W
};

struct DemoApp
{
	HINSTANCE					hInst;					// CX^Xnh
	HWND						hWnd;					// EBhEnh
	D3D_DRIVER_TYPE				driverType;				// hCo[^Cv
	D3D_FEATURE_LEVEL			featureLevel;			// @\x
	ID3D11Device*				pDevice;				// D3D11foCXC^tF[X
	ID3D11DeviceContext*		pDeviceContext;			// D3D11foCXReLXgC^tF[X
	IDXGISwapChain*				pSwapChain;				// Xbv`FC
	ID3D11RenderTargetView*		pRenderTargetView;		// _[^[Qbgr[
	ID3D11VertexShader*			pVS;					// _VF[_
	ID3D11GeometryShader*		pGS;					// WIgVF[_
	ID3D11PixelShader*			pPS;					// sNZVF[_
	ID3D11InputLayout*			pVertexLayout;			// ̓CAEg
	ID3D11Buffer*				pVertexBuffer;			// _obt@
	ID3D11ShaderResourceView*	pTextureResView;		// eNX`
	ID3D11SamplerState*         pSmpState;				// Tv[Xe[g
};

//----------------------------------------------------------------------
// Forward Declarations
//----------------------------------------------------------------------
HRESULT InitWnd( HINSTANCE hInst, int nCmdShow );
HRESULT InitDevice();
void	CleanupDevice();
HRESULT CompileShaderFromFile( TCHAR* szFileName, LPCSTR szEntryPoint, LPCSTR szShaderModel, ID3DBlob** ppBlobOut );
LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wp, LPARAM lp );
void	Render();


//----------------------------------------------------------------------
// Global Variables
//----------------------------------------------------------------------
DemoApp gDemoApp;


//----------------------------------------------------------------------
//! @brief AvP[VCGg[ۃCt
//! @param [in] argc ̐
//! @param [in] argv ̒l
//! @return VXe֒lԋp
//----------------------------------------------------------------------
int main( int argc, char** argv )
{
	HINSTANCE hInst = GetModuleHandle( NULL );

	// EBhE
	if ( FAILED( InitWnd( hInst, SW_SHOWNORMAL ) ) )
	{
		printf( "InitWnd() Failed.\n" );
		return 0;
	}

	// foCX
	if ( FAILED( InitDevice() ) )
	{
		printf( "InitDevice() Failed.\n" );
		return 0;
	}

	// C[v
	MSG msg = { 0 };
	while( WM_QUIT != msg.message )
	{
		if ( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) )
		{
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}
		else
		{
			// `
			Render();
		}
	}

	// foCX̃N[Abv
	CleanupDevice();

	return (int)msg.wParam;
}

//----------------------------------------------------------------------
//! @brief EBhE̐
//! @param [in] hInst CX^Xnh
//! @param [in] nCmdShow 
//----------------------------------------------------------------------
HRESULT InitWnd( HINSTANCE hInst, int nCmdShow )
{
	// gEBhENX̓o^
	WNDCLASSEX wc;
	wc.cbSize = sizeof( WNDCLASSEX );
	wc.style = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc = WndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInst;
	wc.hIcon = LoadIcon( NULL, IDI_APPLICATION );
	wc.hCursor = LoadCursor( NULL, IDC_ARROW );
	wc.hbrBackground = (HBRUSH)( COLOR_WINDOW + 1 );
	wc.lpszMenuName = NULL;
	wc.lpszClassName = TEXT( "AsDemoAppDx11" );
	wc.hIconSm = LoadIcon( NULL, IDI_APPLICATION );
	if ( !RegisterClassEx( &wc ) )
	{
		return E_FAIL;
	}

	// CX^Xnhݒ
	gDemoApp.hInst = hInst;

	// EBhETCY̐ݒ
	RECT rc = { 0, 0, 640, 480 }; // 4 : 3
	//RECT rc = { 0, 0, 800, 450 }; // 16 : 9
	AdjustWindowRect( &rc, WS_OVERLAPPEDWINDOW, FALSE );

	// EBhE̐
	gDemoApp.hWnd = CreateWindow(
		TEXT( "AsDemoAppDx11" ),
		TEXT( "Texture Mapping Sample Program" ),
		WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		rc.right - rc.left,
		rc.bottom - rc.top,
		NULL,
		NULL,
		hInst,
		NULL );
	if ( !gDemoApp.hWnd )
	{
		return E_FAIL;
	}

	// EBhE̕\ݒ
	ShowWindow( gDemoApp.hWnd, nCmdShow );

	return S_OK;
}

//----------------------------------------------------------------------
//! @brief EBhEvV[W
//! @param [in] hWnd EBhEnh
//! @param [in] uMsg bZ[W
//! @param [in] wp
//! @param [in] lp
//----------------------------------------------------------------------
LRESULT CALLBACK WndProc
(
	HWND	hWnd,
	UINT	uMsg,
	WPARAM	wp,
	LPARAM	lp
)
{
	PAINTSTRUCT ps;
	HDC hdc;

	switch( uMsg )
	{
		case WM_PAINT :
			{
				hdc = BeginPaint( hWnd, &ps );
				EndPaint( hWnd, &ps );
			}
			break;

		case WM_DESTROY :
			{
				PostQuitMessage( 0 );
			}
			break;

		default :
			{
				/* DO_NOTHING */
			}
			break;
	}

	return DefWindowProc( hWnd, uMsg, wp, lp );
}

//-----------------------------------------------------------------------
//! @brief t@CVF[_RpC
//! @param [in] szFileName t@C
//! @param [in] szEntryPoint Gg[|Cg
//! @param [in] szShaderMode VF[_vt@C
//! @param [out] ppBlobOut RpCꂽoCgR[h
//-----------------------------------------------------------------------
HRESULT CompileShaderFromFile
(
	TCHAR* szFileName,
	LPCSTR szEntryPoint,
	LPCSTR szShaderModel,
	ID3DBlob** ppBlobOut
)
{
	HRESULT hr = S_OK;

	DWORD dwShaderFlags = D3DCOMPILE_ENABLE_STRICTNESS;
#if defined(DEBUG) || defined(_DEBUG)
	dwShaderFlags |= D3DCOMPILE_DEBUG;
#endif

	ID3DBlob* pErrorBlob;

	// VF[_t@CRpC
	hr = D3DX11CompileFromFile( 
		szFileName,
		NULL,
		NULL,
		szEntryPoint,
		szShaderModel,
		dwShaderFlags,
		0,
		NULL,
		ppBlobOut,
		&pErrorBlob,
		NULL );
	if ( FAILED( hr ) )
	{
		if ( pErrorBlob != NULL )
		{
			// G[bZ[Wo
			OutputDebugStringA( (char*)pErrorBlob->GetBufferPointer() );
		}
	}

	if ( pErrorBlob )
	{
		pErrorBlob->Release();
		pErrorBlob = NULL;
	}

	return hr;
}

//-----------------------------------------------------------------------
//! @brief foCX̏
//-----------------------------------------------------------------------
HRESULT InitDevice()
{
	HRESULT hr = S_OK;

	// EBhETCY擾
	RECT rc;
	GetClientRect( gDemoApp.hWnd, &rc );
	UINT width = rc.right - rc.left;
	UINT height = rc.bottom - rc.top;

	UINT createDeviceFlags = 0;
#if defined(DEBUG) || defined(_DEBUG)
	createDeviceFlags |= D3D11_CREATE_DEVICE_DEBUG;
#endif

	// hCo[^Cv
	D3D_DRIVER_TYPE driverTypes[] = {
		D3D_DRIVER_TYPE_HARDWARE,
		D3D_DRIVER_TYPE_WARP,
		D3D_DRIVER_TYPE_REFERENCE,
	};
	UINT numDriverTypes = sizeof( driverTypes ) / sizeof( driverTypes[0] );

	// 
	D3D_FEATURE_LEVEL featureLevels[] = {
		D3D_FEATURE_LEVEL_11_0,
		D3D_FEATURE_LEVEL_10_1,
		D3D_FEATURE_LEVEL_10_0,
	};
	UINT numFeatureLevels = sizeof( featureLevels ) / sizeof( featureLevels[0] );

	// Xbv`FC̐ݒ
	DXGI_SWAP_CHAIN_DESC sd;
	ZeroMemory( &sd, sizeof(sd) );
	sd.BufferCount = 1;
	sd.BufferDesc.Width = width;
	sd.BufferDesc.Height = height;
	sd.BufferDesc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;
	sd.BufferDesc.RefreshRate.Numerator = 60;
	sd.BufferDesc.RefreshRate.Denominator = 1;
	sd.BufferUsage = DXGI_USAGE_RENDER_TARGET_OUTPUT;
	sd.OutputWindow = gDemoApp.hWnd;
	sd.SampleDesc.Count = 1;
	sd.SampleDesc.Quality = 0;
	sd.Windowed = TRUE;

	// foCXƃXbv`FC쐬
	for( UINT idx = 0; idx < numDriverTypes; idx++ )
	{
		gDemoApp.driverType = driverTypes[ idx ];
		hr = D3D11CreateDeviceAndSwapChain(
			NULL,
			gDemoApp.driverType,
			NULL,
			createDeviceFlags,
			featureLevels,
			numFeatureLevels,
			D3D11_SDK_VERSION,
			&sd,
			&gDemoApp.pSwapChain,
			&gDemoApp.pDevice,
			&gDemoApp.featureLevel,
			&gDemoApp.pDeviceContext );

		if ( SUCCEEDED( hr ) )
		{
			// 烋[vEo
			break;
		}
	}
	if ( FAILED( hr ) )
	{
		// 쐬ɎsI
		return hr;
	}

	// obNobt@擾
	ID3D11Texture2D* pBackBuffer = NULL;
	hr = gDemoApp.pSwapChain->GetBuffer( 0, __uuidof( ID3D11Texture2D ), (LPVOID*)&pBackBuffer );
	if ( FAILED( hr ) )
	{
		return hr;
	}

	// _[^[Qbgr[𐶐
	hr = gDemoApp.pDevice->CreateRenderTargetView( pBackBuffer, NULL, &gDemoApp.pRenderTargetView );
	pBackBuffer->Release();
	pBackBuffer = NULL;
	if ( FAILED( hr ) )
	{
		return hr;
	}

	// o̓}l[WɃ_[^[Qbgr[ݒ
	gDemoApp.pDeviceContext->OMSetRenderTargets( 1, &gDemoApp.pRenderTargetView, NULL ); 

	// r[|[g̐ݒ
	D3D11_VIEWPORT vp;
	vp.Width = (FLOAT)width;
	vp.Height = (FLOAT)height;
	vp.MinDepth = 0.0f;
	vp.MaxDepth = 1.0f;
	vp.TopLeftX = 0;
	vp.TopLeftY = 0;
	gDemoApp.pDeviceContext->RSSetViewports( 1, &vp );

	// _VF[_RpC
	ID3DBlob* pVSBlob = NULL;
	hr = CompileShaderFromFile( TEXT("Contents/Shaders/Sample.hlsl"), "VSFunc", "vs_4_0", &pVSBlob );
	if ( FAILED( hr ) )
	{
		MessageBox( NULL,
			TEXT( "he FX file cannot be compiled.  Please run this executable from the directory that contains the FX file." ),
			TEXT( "Shader Compile Error" ),
			MB_OK | MB_ICONERROR );
		return hr;
	}

	// _VF[_
	hr = gDemoApp.pDevice->CreateVertexShader( pVSBlob->GetBufferPointer(), pVSBlob->GetBufferSize(), NULL, &gDemoApp.pVS );
	if ( FAILED( hr ) )
	{
		pVSBlob->Release();
		pVSBlob = NULL;
		return hr;
	}

	// ̓CAEg̒`
	D3D11_INPUT_ELEMENT_DESC layout[] = {
		{ "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D11_INPUT_PER_VERTEX_DATA, 0 },
		{ "TEXCOORD", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 12, D3D11_INPUT_PER_VERTEX_DATA, 0 },
	};
	UINT numElements = sizeof( layout ) / sizeof( layout[0] );

	// ̓CAEg𐶐
	hr = gDemoApp.pDevice->CreateInputLayout(
		layout,
		numElements,
		pVSBlob->GetBufferPointer(),
		pVSBlob->GetBufferSize(),
		&gDemoApp.pVertexLayout );
	pVSBlob->Release();
	pVSBlob = NULL;
	if ( FAILED( hr ) )
	{
		return hr;
	}

	// ̓AZuɓ̓CAEgݒ
	gDemoApp.pDeviceContext->IASetInputLayout( gDemoApp.pVertexLayout );

	// WIgVF[_RpC
	ID3DBlob* pGSBlob = NULL;
	hr = CompileShaderFromFile( TEXT("Contents/Shaders/Sample.hlsl"), "GSFunc", "gs_4_0", &pGSBlob );
	if ( FAILED( hr ) )
	{
		MessageBox(
			NULL,
			TEXT( "The FX file cannot be compiled.  Please run this executable from the directory that contains the FX file." ),
			TEXT( "Shader Compile Error" ),
			MB_OK | MB_ICONERROR );
		return hr;
	}

	// WIgVF[_𐶐
	hr = gDemoApp.pDevice->CreateGeometryShader(
		pGSBlob->GetBufferPointer(),
		pGSBlob->GetBufferSize(),
		NULL,
		&gDemoApp.pGS );
	pGSBlob->Release();
	pGSBlob = NULL;
	if ( FAILED( hr ) )
	{
		return hr;
	}

	// sNZVF[_RpC
	ID3DBlob* pPSBlob = NULL;
	hr = CompileShaderFromFile( TEXT("Contents/Shaders/Sample.hlsl"), "PSFunc", "ps_4_0", &pPSBlob );
	if ( FAILED( hr ) )
	{
		MessageBox(
			NULL,
			TEXT( "The FX file cannot be compiled.  Please run this executable from the directory that contains the FX file." ),
			TEXT( "Shader Compile Error" ),
			MB_OK | MB_ICONERROR );
		return hr;
	}

	// sNZVF[_
	hr = gDemoApp.pDevice->CreatePixelShader(
		pPSBlob->GetBufferPointer(),
		pPSBlob->GetBufferSize(),
		NULL,
		&gDemoApp.pPS );
	pPSBlob->Release();
	pPSBlob = NULL;
	if ( FAILED( hr ) )
	{
		return hr;
	}

	// _̒`
	CustomVertex vertices[] = {
		{ XMFLOAT3( 0.0f, 0.5f, 0.5f ),		XMFLOAT2( 0.5f, 0.0f ) }, 
		{ XMFLOAT3( 0.5f, -0.5f, 0.5f ),	XMFLOAT2( 1.0f, 1.0f ) },
		{ XMFLOAT3( -0.5f, -0.5f, 0.5f ),	XMFLOAT2( 0.0f, 1.0f ) },
	};

	// _obt@̐ݒ
	D3D11_BUFFER_DESC bd;
	ZeroMemory( &bd, sizeof(bd) );
	bd.Usage = D3D11_USAGE_DEFAULT;
	bd.ByteWidth = sizeof( CustomVertex ) * 3;
	bd.BindFlags = D3D11_BIND_VERTEX_BUFFER;
	bd.CPUAccessFlags = 0;

	// Tu\[X̐ݒ
	D3D11_SUBRESOURCE_DATA initData;
	ZeroMemory( &initData, sizeof(initData ) );
	initData.pSysMem = vertices;

	// _obt@
	hr = gDemoApp.pDevice->CreateBuffer( &bd, &initData, &gDemoApp.pVertexBuffer );
	if ( FAILED( hr ) )
	{
		return hr;
	}

	// ̓AZuɒ_obt@ݒ
	UINT stride = sizeof( CustomVertex );
	UINT offset = 0;
	gDemoApp.pDeviceContext->IASetVertexBuffers( 0, 1, &gDemoApp.pVertexBuffer, &stride, &offset );

	// v~eBu̎ނݒ
	gDemoApp.pDeviceContext->IASetPrimitiveTopology( D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST );

	// eNX`[h
	hr = D3DX11CreateShaderResourceViewFromFile( gDemoApp.pDevice,
		TEXT( "Contents/Textures/tiger.dds" ),
		NULL,
		NULL,
		&gDemoApp.pTextureResView,
		NULL );
	if ( FAILED( hr ) )
	{
		printf( "Error : Texture Load Failed.\n" );
		return hr;
	}

	// eNX`]
	gDemoApp.pDeviceContext->PSSetShaderResources( 0, 1, &gDemoApp.pTextureResView );

#if 1
	// Tv[Xe[g̐ݒ
	D3D11_SAMPLER_DESC smpDesc;
	ZeroMemory( &smpDesc, sizeof(smpDesc) );
	smpDesc.Filter = D3D11_FILTER_MIN_MAG_MIP_LINEAR;
	smpDesc.AddressU = D3D11_TEXTURE_ADDRESS_WRAP;
	smpDesc.AddressV = D3D11_TEXTURE_ADDRESS_WRAP;
	smpDesc.AddressW = D3D11_TEXTURE_ADDRESS_WRAP;
	smpDesc.ComparisonFunc = D3D11_COMPARISON_NEVER;
	smpDesc.MinLOD = 0;
	smpDesc.MaxLOD = D3D11_FLOAT32_MAX;

	// Tv[Xe[g
	hr = gDemoApp.pDevice->CreateSamplerState( &smpDesc, &gDemoApp.pSmpState );
	if ( FAILED( hr ) )
	{
		printf( "Error : CreateSamplerState() Failed.\n" );
		return hr;
	}
	
	// Tv[Xe[g]
	gDemoApp.pDeviceContext->PSSetSamplers( 0, 1, &gDemoApp.pSmpState );
#endif
	
	return S_OK;
}

//-----------------------------------------------------------------------
//! @brief foCX̃N[Abv
//-----------------------------------------------------------------------
void CleanupDevice()
{
	if ( gDemoApp.pDeviceContext )
	{
		gDemoApp.pDeviceContext->ClearState();
	}
	if ( gDemoApp.pVertexBuffer )
	{
		gDemoApp.pVertexBuffer->Release();
		gDemoApp.pVertexBuffer = NULL;
	}
	if ( gDemoApp.pVertexLayout )
	{
		gDemoApp.pVertexLayout->Release();
		gDemoApp.pVertexLayout = NULL;
	}
	if ( gDemoApp.pVS )
	{
		gDemoApp.pVS->Release();
		gDemoApp.pVS = NULL;
	}
	if ( gDemoApp.pPS )
	{
		gDemoApp.pPS->Release();
		gDemoApp.pPS = NULL;
	}
	if ( gDemoApp.pRenderTargetView )
	{
		gDemoApp.pRenderTargetView->Release();
		gDemoApp.pRenderTargetView = NULL;
	}
	if ( gDemoApp.pSwapChain )
	{
		gDemoApp.pSwapChain->Release();
		gDemoApp.pSwapChain = NULL;
	}
	if ( gDemoApp.pDeviceContext )
	{
		gDemoApp.pDeviceContext->Release();
		gDemoApp.pDeviceContext = NULL;
	}
	if ( gDemoApp.pDevice )
	{
		gDemoApp.pDevice->Release();
		gDemoApp.pDevice = NULL;
	}
}

//-----------------------------------------------------------------------
//! @brief t[`
//-----------------------------------------------------------------------
void Render()
{
	// _[^[Qbgr[NA
	float clearColor[ 4 ] = { 0.3f, 0.3f, 1.0f, 1.0f };
	gDemoApp.pDeviceContext->ClearRenderTargetView( gDemoApp.pRenderTargetView, clearColor );

	// VF[_ݒ肵ĕ`
	gDemoApp.pDeviceContext->VSSetShader( gDemoApp.pVS, NULL, 0 );
	gDemoApp.pDeviceContext->GSSetShader( gDemoApp.pGS, NULL, 0 );
	gDemoApp.pDeviceContext->PSSetShader( gDemoApp.pPS, NULL, 0 );
	gDemoApp.pDeviceContext->Draw( 3, 0 );

	// tbv
	gDemoApp.pSwapChain->Present( 0, 0 );		
}